﻿/*
  In App.xaml:
  <Application.Resources>
      <vm:ExampleViewModelLocator xmlns:vm="clr-namespace:MVVMLightToolkit.ViewModel"
                                   x:Key="Locator" />
  </Application.Resources>
  
  In the View:
  DataContext="{Binding Source={StaticResource Locator}, Path=ViewModelName}"
  
  OR (WPF only):
  
  xmlns:vm="clr-namespace:MVVMLightToolkit.ViewModel"
  DataContext="{Binding Source={x:Static vm:ExampleViewModelLocator.ViewModelNameStatic}}"
*/

namespace MVVMLightToolkit.ViewModel
{
    /// <summary>
    /// This class contains static references to all the view models in the
    /// application and provides an entry point for the bindings.
    /// <para>
    /// Use the <strong>mvvmlocatorproperty</strong> snippet to add ViewModels
    /// to this locator.
    /// </para>
    /// <para>
    /// In Silverlight and WPF, place the ExampleViewModelLocator in the App.xaml resources:
    /// </para>
    /// <code>
    /// &lt;Application.Resources&gt;
    ///     &lt;vm:ExampleViewModelLocator xmlns:vm="clr-namespace:MVVMLightToolkit.ViewModel"
    ///                                  x:Key="Locator" /&gt;
    /// &lt;/Application.Resources&gt;
    /// </code>
    /// <para>
    /// Then use:
    /// </para>
    /// <code>
    /// DataContext="{Binding Source={StaticResource Locator}, Path=ViewModelName}"
    /// </code>
    /// <para>
    /// You can also use Blend to do all this with the tool's support.
    /// </para>
    /// <para>
    /// See http://www.galasoft.ch/mvvm/getstarted
    /// </para>
    /// <para>
    /// In <strong>*WPF only*</strong> (and if databinding in Blend is not relevant), you can delete
    /// the ViewModelName property and bind to the ViewModelNameStatic property instead:
    /// </para>
    /// <code>
    /// xmlns:vm="clr-namespace:MVVMLightToolkit.ViewModel"
    /// DataContext="{Binding Source={x:Static vm:ExampleViewModelLocator.ViewModelNameStatic}}"
    /// </code>
    /// </summary>
    public class ViewModelLocator
    {
        #region ExampleVM Locator Property
        private static ExampleViewModel _exampleViewModelLocator;

        /// <summary>
        /// Gets the Example property.
        /// </summary>
        public static ExampleViewModel ExampleStatic
        {
            get
            {
                if (_exampleViewModelLocator == null)
                {
                    CreateExample();
                }

                return _exampleViewModelLocator;
            }
        }

        /// <summary>
        /// Gets the ExampleVM property.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance",
            "CA1822:MarkMembersAsStatic",
            Justification = "This non-static member is needed for data binding purposes.")]
        public ExampleViewModel Example
        {
            get
            {
                return ExampleStatic;
            }
        }

        /// <summary>
        /// Provides a deterministic way to delete the ExampleVM property.
        /// </summary>
        public static void ClearExample()
        {
            _exampleViewModelLocator.Cleanup();
            _exampleViewModelLocator = null;
        }

        /// <summary>
        /// Provides a deterministic way to create the ExampleVM property.
        /// </summary>
        public static void CreateExample()
        {
            if (_exampleViewModelLocator == null)
            {
                _exampleViewModelLocator = new ExampleViewModel();
            }
        }

        /// <summary>
        /// Cleans up all the resources.
        /// </summary>
        public static void Cleanup()
        {
            ClearExample();
        } 
        #endregion

        /// <summary>
        /// Initializes a new instance of the ExampleViewModelLocator class.
        /// </summary>
        public ViewModelLocator()
        {
            ////if (ViewModelBase.IsInDesignModeStatic)
            ////{
            ////    // Create design time view models
            ////}
            ////else
            ////{
            ////    // Create run time view models
            ////}
        }
    }
}